@@ -1,17 +1,16 @@
+from pyield.anbima.difusao import tpf_difusao
 from pyield.anbima.ettj_intraday import intraday_ettj
 from pyield.anbima.ettj_last import last_ettj
 from pyield.anbima.ima import last_ima
 from pyield.anbima.imaq import imaq
-from pyield.anbima.ipca import ipca_projection
-from pyield.anbima.tpf import _fetch_tpf_data, tpf_data, tpf_fixed_rate_maturities
+from pyield.anbima.tpf import tpf_data, tpf_maturities
 
 __all__ = [
     "last_ima",
     "imaq",
-    "ipca_projection",
-    "_fetch_tpf_data",
     "tpf_data",
-    "tpf_fixed_rate_maturities",
+    "tpf_maturities",
     "last_ettj",
     "intraday_ettj",
+    "tpf_difusao",
 ]
