@@ -36,15 +36,18 @@ def forwards(
         - tₖ é o prazo em anos para o vértice atual (calculado como duₖ/252).
         - A constante 252 representa o número de dias úteis no ano.
 
+    A função preserva a ordem original dos dados de entrada e lida com valores nulos
+    de forma apropriada. Valores nulos nas entradas resultarão em valores nulos
+    nas taxas a termo calculadas.
+
     A primeira taxa a termo de cada grupo é definida como a
     taxa zero desse primeiro vértice (fwd₁ = r₁), dado que não existe um vértice
     anterior a r₁ para se calcular a taxa a termo no primeiro ponto.
 
-    Valores nulos nas taxas ou prazos de entrada resultarão em valores nulos
-    nas taxas a termo calculadas. A função também lida com agrupamentos
-    opcionais, permitindo calcular taxas a termo para diferentes grupos de
-    datas. O agrupamento é feito com base em `group_by`. Se este
-    argumento for None, todos os dados serão tratados como um único grupo.
+    A função também lida com agrupamentos opcionais, permitindo calcular taxas
+    a termo para diferentes grupos de datas. O agrupamento é feito com base em `group_by`.
+    Se este argumento for None, todos os dados serão tratados como um único grupo.
+
     A função calcula as taxas a termo para todos os pontos, exceto o primeiro
     de cada grupo, que é tratado separadamente.
 
@@ -141,6 +144,8 @@ def forwards(
         - A função ordena os dados de entrada primeiro por `group_by`,
         se for fornecido, e depois por `bdays` para garantir a ordem cronológica
         correta no cálculo das taxas a termo.
+        - Valores nulos em `bdays` ou `rates` são ignorados no cálculo,
+        resultando em valores nulos nas posições correspondentes na saída.
         - Os resultados são retornados na mesma ordem dos dados de entrada.
     """  # noqa: E501
     # Validações iniciais
@@ -164,7 +169,7 @@ def forwards(
     exp3 = 1 / (pl.col("time_k") - pl.col("time_j"))  # 1/(tₖ - tⱼ)
     fwd_formula = (exp1 / exp2) ** exp3 - 1
 
-    # --- Início da Lógica com Expressões (Lazy API) ---
+    # 4. Calcular as taxas a termo
     df_fwd = (
         df_orig.drop_nans()
         .drop_nulls()
@@ -178,20 +183,24 @@ def forwards(
         )
         .with_columns(fwd=fwd_formula)
         .with_columns(
-            # Usar a taxa spot para a primeira entrada de cada grupo
-            fwd=pl.when(pl.col("time_j").is_null())
+            # A matriz de cálculo já foi tratada: ela está deduplicada,
+            # sem nulos e ordenada por group_by e du_k. Então, basta
+            # ajustar a primeira taxa fwd de cada grupo para ser igual à taxa spot!
+            fwd=pl.when(pl.col("du_k") == pl.first("du_k").over("group_by"))
             .then(pl.col("rate_k"))
             .otherwise(pl.col("fwd"))
         )
     )
-    s_fwd = df_orig.join(
-        df_fwd,
+    # 5. Reunir os resultados na ordem original
+    df_orig = df_orig.join(
+        df_fwd.drop("rate_k"),  # rate_k já existe em df_orig
         on=["du_k", "group_by"],
         how="left",
         maintain_order="left",
-    ).get_column("fwd")
+    )
 
-    return s_fwd
+    # Retornar a série de taxas a termo
+    return df_orig["fwd"]
 
 
 def forward(
@@ -199,7 +208,7 @@ def forward(
     bday2: int,
     rate1: float,
     rate2: float,
-) -> float | None:
+) -> float:
     r"""
     Calcula a taxa a termo (forward rate) entre dois prazos (dias úteis).
 
@@ -224,7 +233,7 @@ def forward(
         rate2 (float): Taxa zero (spot rate) para o prazo `bday2`.
 
     Returns:
-        float | None: A taxa a termo calculada entre `bday1` e `bday2`. Retorna
+        float: A taxa a termo calculada entre `bday1` e `bday2`. Retorna
             `None` se `bday1 >= bday2` ou se qualquer um dos
             argumentos de entrada for float('nan') ou None.
 
@@ -234,11 +243,11 @@ def forward(
         0.0700952380952371
         >>> # Exemplo inválido: bday1 >= bday2
         >>> print(yd.forward(20, 10, 0.06, 0.05))
-        None
+        nan
 
-        >>> # Argumentos nulos retornam None
+        >>> # Argumentos nulos retornam nan
         >>> print(yd.forward(10, 20, 0.05, None))
-        None
+        nan
 
     Note:
         `bday2` precisa ser necessariamente maior que `bday1` para que
@@ -251,11 +260,11 @@ def forward(
     """  # noqa: E501
     if has_null_args(rate1, rate2, bday1, bday2):
         # If any of the inputs are nullable, return None
-        return None
+        return float("nan")
 
     # Handle the case where the two dates are the same
     if bday2 <= bday1:
-        return None
+        return float("nan")
 
     # Convert business days to business years
     t1 = bday1 / 252
