@@ -81,7 +81,8 @@ def payment_dates(
         maturity (DateScalar): The maturity date.
 
     Returns:
-        pl.Series: Series of coupon dates within the specified range.
+        pl.Series: Series of coupon dates within the specified range. Returns an
+            empty Series if the maturity date is before the settlement date.
 
     Examples:
         >>> from pyield import ntnc
@@ -110,7 +111,7 @@ def payment_dates(
 
     # Check if maturity date is after the start date
     if maturity < settlement:
-        raise ValueError("Maturity date must be after the start date.")
+        return pl.Series(dtype=pl.Date)
 
     # Initialize loop variables
     coupon_date = maturity
@@ -176,6 +177,10 @@ def cash_flows(
     # Get the coupon dates between the settlement and maturity dates
     pay_dates = payment_dates(settlement, maturity)
 
+    # Return empty DataFrame if no payment dates (settlement >= maturity)
+    if pay_dates.is_empty():
+        return pl.DataFrame(schema={"PaymentDate": pl.Date, "CashFlow": pl.Float64})
+
     # Get the right coupon payment and final payment values
     coupon_pmt = _get_coupon_pmt(maturity)
     final_pmt = _get_final_pmt(maturity)
@@ -194,7 +199,7 @@ def quotation(
     settlement: DateScalar,
     maturity: DateScalar,
     rate: float,
-) -> float | None:
+) -> float:
     """
     Calculate the NTN-C quotation in base 100 using Anbima rules.
 
@@ -205,7 +210,7 @@ def quotation(
             the cash flows, which is the yield to maturity (YTM) of the NTN-C.
 
     Returns:
-        float | None: The NTN-C quotation truncated to 4 decimal places.
+        float: The NTN-C quotation truncated to 4 decimal places.
 
     References:
         - https://www.anbima.com.br/data/files/A0/02/CC/70/8FEFC8104606BDC8B82BA2A8/Metodologias%20ANBIMA%20de%20Precificacao%20Titulos%20Publicos.pdf
@@ -220,7 +225,7 @@ def quotation(
     """
     # Validate and normalize inputs
     if has_null_args(settlement, maturity, rate):
-        return None
+        return float("nan")
     settlement = cv.convert_dates(settlement)
     maturity = cv.convert_dates(maturity)
 
@@ -246,7 +251,7 @@ def quotation(
 def price(
     vna: float,
     quotation: float,
-) -> float | None:
+) -> float:
     """
     Calculate the NTN-C price using Anbima rules.
 
@@ -257,7 +262,7 @@ def price(
         quotation (float): The NTN-C quotation in base 100.
 
     Returns:
-        float | None: The NTN-C price truncated to 6 decimal places.
+        float: The NTN-C price truncated to 6 decimal places.
 
     References:
         - https://www.anbima.com.br/data/files/A0/02/CC/70/8FEFC8104606BDC8B82BA2A8/Metodologias%20ANBIMA%20de%20Precificacao%20Titulos%20Publicos.pdf
@@ -268,7 +273,7 @@ def price(
         8347.348705
     """
     if has_null_args(vna, quotation):
-        return None
+        return float("nan")
     return tl.truncate(vna * quotation / 100, 6)
 
 
@@ -276,7 +281,7 @@ def duration(
     settlement: DateScalar,
     maturity: DateScalar,
     rate: float,
-) -> float | None:
+) -> float:
     """
     Calculate the Macaulay duration of the NTN-C bond in business years.
 
@@ -286,7 +291,7 @@ def duration(
         rate (float): The discount rate used to calculate the duration.
 
     Returns:
-        float | None: The Macaulay duration of the NTN-C bond in business years.
+        float: The Macaulay duration of the NTN-C bond in business years.
 
     Examples:
         >>> from pyield import ntnc
@@ -295,7 +300,7 @@ def duration(
     """
     # Validate and normalize inputs
     if has_null_args(settlement, maturity, rate):
-        return None
+        return float("nan")
     settlement = cv.convert_dates(settlement)
     maturity = cv.convert_dates(maturity)
 
